@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -25,9 +27,9 @@ inline
 void
 op_hist::apply_noalias(Mat<uword>& out, const Mat<eT>& A, const uword n_bins, const uword dim)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check( ((A.is_vec() == false) && (A.is_empty() == false)), "hist(): only vectors are supported when automatically determining bin centers" );
+  arma_conform_check( ((A.is_vec() == false) && (A.is_empty() == false)), "hist(): only vectors are supported when automatically determining bin centers" );
   
   if(n_bins == 0)  { out.reset(); return; }
   
@@ -67,7 +69,7 @@ op_hist::apply_noalias(Mat<uword>& out, const Mat<eT>& A, const uword n_bins, co
   if(arma_isfinite(min_val) == false) { min_val = priv::most_neg<eT>(); }
   if(arma_isfinite(max_val) == false) { max_val = priv::most_pos<eT>(); }
   
-  Col<eT> c(n_bins);
+  Col<eT> c(n_bins, arma_nozeros_indicator());
   eT* c_mem = c.memptr();
   
   for(uword ii=0; ii < n_bins; ++ii)
@@ -87,7 +89,7 @@ inline
 void
 op_hist::apply(Mat<uword>& out, const mtOp<uword, T1, op_hist>& X)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   const uword n_bins = X.aux_uword_a;
   
